#region License
/*
This file is part of Screen Position.

Crystal Runner is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Screen Position is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Crystal Runner; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#endregion License

#region Includes
using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Runtime.InteropServices;
using System.Windows.Forms;
#endregion Includes

namespace ScreenPosition
{
  public partial class Viewer : Form
  {
    #region Raster Operations Enumeration
    public enum RasterOperations
    {
      SRCCOPY = 0x00CC0020, // dest = source
      SRCPAINT = 0x00EE0086, // dest = source OR dest
      SRCAND = 0x008800C6, // dest = source AND dest
      SRCINVERT = 0x00660046, // dest = source XOR dest
      SRCERASE = 0x00440328, // dest = source AND (NOT dest)
      NOTSRCCOPY = 0x00330008, // dest = (NOT source)
      NOTSRCERASE = 0x001100A6, // dest = (NOT src) AND (NOT dest)
      MERGECOPY = 0x00C000CA, // dest = (source AND pattern)
      MERGEPAINT = 0x00BB0226, // dest = (NOT source) OR dest
      PATCOPY = 0x00F00021, // dest = pattern
      PATPAINT = 0x00FB0A09, // dest = DPSnoo
      PATINVERT = 0x005A0049, // dest = pattern XOR dest
      DSTINVERT = 0x00550009, // dest = (NOT dest)
      BLACKNESS = 0x00000042, // dest = BLACK
      WHITENESS = 0x00FF0062, // dest = WHITE
    };
    #endregion Raster Operations Enumeration

    #region GDI32 Imports
    // Define external Win32 functions.
    [DllImport("gdi32.dll")]
    public static extern IntPtr CreateDC(string strDriver, string strDevice,
                                         string strOutput, IntPtr pData);
    [DllImport("gdi32.dll")]
    public static extern bool DeleteDC(IntPtr hdc);

    [DllImport("gdi32.dll")]
    public static extern int GetPixel(IntPtr hdc, int x, int y);

    [DllImport("GDI32.dll")]
    public static extern bool BitBlt(int hdcDest, int nXDest, int nYDest,
                                     int nWidth, int nHeight, int hdcSrc,
                                     int nXSrc, int nYSrc, int dwRop);
    #endregion GDI Imports

    #region User32 Imports
    [DllImport("User32.dll")]
    public static extern int GetDesktopWindow();

    [DllImport("User32.dll")]
    public static extern int GetWindowDC(int hWnd);

    [DllImport("User32.dll")]
    public static extern int ReleaseDC(int hWnd, int hDC);
    #endregion User32 Imports

    #region Private Members
    private Color clr;
    #endregion Private Members

    #region Consturctor
    public Viewer()
    {
      InitializeComponent();
    }
    #endregion Consturctor

    #region Form1_Load
    private void Form1_Load(object sender, EventArgs e)
    {
      Timer timer = new Timer();
      timer.Interval = 100;
      timer.Tick += new EventHandler(TimerOnTick);
      timer.Start();
    }
    #endregion Form1_Load

    #region Timer OnTick
    void TimerOnTick(object obj, EventArgs ea)
    {
      Point pt = MousePosition;

      IntPtr hdcScreen = CreateDC("Display", null, null, IntPtr.Zero);
      int cr = GetPixel(hdcScreen, pt.X, pt.Y);

      lblX.Text = pt.X.ToString();
      lblY.Text = pt.Y.ToString();

      

      clr = Color.FromArgb((cr & 0x000000FF),
                           (cr & 0x0000FF00) >> 8,
                           (cr & 0x00FF0000) >> 16);


      PaintMaginifier(hdcScreen, pt.X, pt.Y);
      
      DeleteDC(hdcScreen);

      string str = String.Format("{0:X2}-{1:X2}-{2:X2}\n{3}-{4}-{5}",
                                 clr.R, clr.G, clr.B, clr.R, clr.G, clr.B);

      lblColor.Text = str;
      
    }
    #endregion Timer OnTick

    #region Paint Maginifier
    private void PaintMaginifier(IntPtr hdcScreen, int x, int y)
    {
        Graphics e = CreateGraphics();

        float percent = vsbMagnifier.Value / 100f;
        float lengthX = 255f * percent;
        float lengthY = 120f * percent;

        toolStripStatusLabel1.Text = "Zoom Factor : " + (100 - vsbMagnifier.Value) + "X";

        float offsetX = lengthX / 2;
        float offsetY = lengthY / 2;

        int areaX = 255;
        int areaY = 120;

        Bitmap b = new Bitmap(areaX, areaY);
        Graphics g = Graphics.FromImage(b);

        g.SmoothingMode = SmoothingMode.HighSpeed;
        g.CompositingQuality = CompositingQuality.HighSpeed;
        g.PixelOffsetMode = PixelOffsetMode.HighSpeed;
        g.InterpolationMode = InterpolationMode.Low;

        e.SmoothingMode = SmoothingMode.HighSpeed;
        e.CompositingQuality = CompositingQuality.HighSpeed;
        e.PixelOffsetMode = PixelOffsetMode.HighSpeed;
        e.InterpolationMode = InterpolationMode.Low;

        IntPtr hdcGraphics = g.GetHdc();

        BitBlt(hdcGraphics.ToInt32(), 0, 0, areaX, areaY, hdcScreen.ToInt32(), x - (int)offsetX, y - (int)offsetY, (int)RasterOperations.SRCCOPY);

        g.ReleaseHdc(hdcGraphics);

        e.DrawImage(b, new Rectangle(10, 47, areaX, areaY), 0, 0, lengthX, lengthY, GraphicsUnit.Pixel);

        e.Dispose();
      }
    #endregion Paint Maginifier
    }
}